import os
import sys

import _winreg

class RVars(object):
    """
    For rpy2 to function properly on win32, it appears that the following 
    variables need to be available in the environment: R_HOME, R_USER. 
    In addition, the directory containing the R binaries should be in the PATH variable.
    These variables should be persistent, so they should be written to the registry.
    When writing values to the registry, we'll try to write to the LOCAL_MACHINE hive in preference to the LOCAL_USER hive
    """

    # depending on whether the user has administrator permission, the keys should be in either of these hives
    MACHINE_HIVE = _winreg.HKEY_LOCAL_MACHINE
    USER_HIVE = _winreg.HKEY_CURRENT_USER

    # when installed, if the user chose this option, R wrote information to the registry in this key
    R_KEY = "Software\\R-core\\R"
    
    # environmental variable information should be in this key
    ENV_KEY = "SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment"

    def set_vars(self):
        env_vars = self._get_env_vars()
        reg_vars = self._get_reg_vars()
        # make sure that R_USER is defined persistently
        # if it isn't found, set it to the user's home directory
        R_USER, HOME_DIR = env_vars['R_USER'], env_vars['HOME_DIR']
        if not R_USER:
            if not HOME_DIR:
                print 'The home directory for the user was not found in the registry. The variable R_USER should be set manually.'
            else:
                try:
                    self._add_reg_value(MACHINE_HIVE, ENV_KEY, 'R_USER', HOME_DIR)    
                except WindowsError:
                    self._add_reg_value(LOCAL_HIVE, ENV_KEY, 'R_USER', HOME_DIR)                        
                
        # make sure that R_HOME is defined persistently
        R_HOME, R_ROOT = env_vars['R_HOME'], reg_vars['R_ROOT']
        if not env_vars['R_HOME']:
            if not R_ROOT:
                print 'The location of R was not found in the registry. The variable R_HOME should be set manually.'
            else:
                try:
                    self._add_reg_value(MACHINE_HIVE, ENV_KEY, 'R_HOME', R_ROOT)
                except WindowsError:
                    self._add_reg_value(USER_HIVE, ENV_KEY, 'R_HOME', R_ROOT)                
                    
        # make sure that the path to the R binaries is stored in the PATH persistently
        # get the path to the R binaries      
        R_BIN_DIR = reg_vars['R_BIN_DIR']
        if not R_BIN_DIR:
            print 'The install location for R was not found in the registry. The directory path to the R binaries should be added to the PATH variable by hand.'
        else:
            # is the directory already in the path?
            try:
                if not self._is_inpath(MACHINE_HIVE, R_BIN_DIR):
                    self._append_to_path(R_BIN_DIR, hive=MACHINE_HIVE, key=ENV_KEY)
            except WindowsError:
                if not self._is_inpath(USER_HIVE, R_BIN_DIR):
                    self._append_to_path(R_BIN_DIR, hive=USER_HIVE, key=ENV_KEY)
        
    def _set_reg_value(self, hive, key, value_name, value, value_type=_winreg.REG_SZ):
        reg = _winreg.ConnectRegistry(None, hive)
        k = _winreg.OpenKey(reg, key, 0, _winreg.KEY_WRITE)
        _winreg.SetValueEx(k, value_name, 0, value_type, value)
        _winreg.CloseKey(k)

    def _get_reg_value(self, hive, key, value_name):
        reg = _winreg.ConnectRegistry(None, hive)
        k = _winreg.OpenKey(reg, key, 0, _winreg.KEY_ALL_ACCESS)
        value_data = _winreg.QueryValueEx(k, value_name)   # returns a tuple (value, registry type of this value). 
        _winreg.CloseKey(k)
        return value_data[0]
        
    def _add_reg_value(self, hive, key, value_name, value_data):
        self._set_reg_value(hive, key, value_name, value_data, value_type=_winreg.REG_EXPAND_SZ)   
        
    def _get_path_data(self, hive, key=ENV_KEY):    
        path = self._get_reg_value(hive, key, "Path") 
        return path
        
    def _append_to_path(self, dir, hive, key=ENV_KEY):
        reg = _winreg.ConnectRegistry(None, hive)
        base_path = self._get_path_data(hive, key=key)
        path = base_path + ";" + dir
        self._set_reg_value(reg, key, "Path", path, _winreg.REG_EXPAND_SZ)

    def _is_inpath(self, hive, target_path, key=ENV_KEY):
        reg_path = self._get_path_data(hive, key=key)
        tp = self._normpath(target_path)
        path_parts = map(self._normpath, reg_path.split(';'))
        inpath = tp in path_parts
        return inpath

    def _get_env_vars(self):
        osget = os.environ.get
        home_drive = osget('HOMEDRIVE', None) 
        home_path = osget('HOMEPATH', None)
        if home_drive and home_path:
            hdir = os.path.join(home_drive, home_path)
        else:
            hdir = None
        vars = dict(R_HOME = osget('R_HOME', None), 
                    R_USER = osget('R_USER', None),
                    HOME_DIR = hdir)
        return vars
        
    def _get_reg_vars(self, hive, key=R_KEY):
        R_bindir = None
        try:
            R_obj = _winreg.OpenKey(hive, key)
            R_ver = _winreg.EnumKey(R_obj, 0)
            R_rt = self._get_reg_value(hive, key + '\\' + R_ver, "InstallPath")
        except WindowsError:
            R_ver, R_rt = None, None
        if R_rt:
            # assume that the user wants the R binaries with the same 'bitness' as the version of Python he/she is running
            arch = 'x64' if (sys.maxsize > 2**32) else 'i386'
            R_bindir = os.path.join(R_rt, 'bin', arch)
        vars = dict(R_VERSION = R_ver, 
                    R_ROOT = R_rt,
                    R_BIN_DIR = R_bindir)
        return vars
          
    def _normpath(self, x):
        npath = os.path.normcase(os.path.abspath(x.strip()))
        return npath
        
if __name__ == '__main__':
    rvr = RVars()
    rvr.set_vars()
